@@ -160,7 +160,8 @@ as.mcmc.emmGrid = function(x, names = TRUE, sep.chains = TRUE,
     }
     if(is.null(dimnames(mat)))
         dimnames(mat) = list(seq_len(nrow(mat)), seq_len(ncol(mat)))
-    dimnames(mat)[[2]] = do.call(paste, c(unname(x@grid[, nm, drop = FALSE]), sep=", "))
+    dimnames(mat)[[2]] = do.call(paste, c(unname(x@grid[, nm, drop = FALSE]), 
+                                          sep = get_emm_option("sep")))
     n.chains = attr(x@post.beta, "n.chains")
     if (!sep.chains || is.null(n.chains) || (n.chains == 1))
         coda::mcmc(mat)
@@ -224,7 +225,7 @@ as.mcmc.list.emm_list = function(x, which = 1, ...) {
 #'   \code{sigma} to exist in the object or be specified.
 #' @param sigma Error SD assumed for bias correction (when 
 #'   \code{type = "response"}. If not specified,
-#'   \code{object@misc$sigma} is used, and an error is thrown if it is not found.
+#'   \code{object@misc$sigma} is used, and a warning if it is not found or invalid.
 #'   \emph{Note:} \code{sigma} may be a vector, as long as it conforms to the 
 #'   number of observations in the posterior sample.
 #' @param ... required but not used
@@ -249,7 +250,8 @@ as.mcmc.list.emm_list = function(x, which = 1, ...) {
 #'   A closely related quantity is the ROPE (region of practical equivalence),
 #'   obtainable via \code{bayestestR::rope(object, range = c(-delta, delta))}.
 #'   Its value is approximately \code{100 * p.equiv / 0.95} if the default
-#'   \code{ci = 0.95} is used.
+#'   \code{ci = 0.95} is used. See also \pkg{bayestestR}'s 
+#'   \href{https://github.com/easystats/bayestestR/issues/567}{issue #567}.
 #'   
 #'   Finally, a Bayes factor for equivalence is obtainable by dividing 
 #'   \code{odds.eq} by the prior odds of equivalence, assessed or elicited separately.
@@ -341,6 +343,7 @@ hpd.summary = function(object, prob, by, type, point.est = median,
             if (missing(sigma))
                 sigma = object@misc@sigma
             link = .make.bias.adj.link(link, sigma)
+            bias.adjust = attr(link, "bias.adjust")
         }
         
         for (j in seq_along(mcmc[1, ]))
@@ -398,6 +401,7 @@ hpd.summary = function(object, prob, by, type, point.est = median,
 
 
 # Currently, data is required, as call is not stored
+#' @exportS3Method recover_data MCMCglmm
 recover_data.MCMCglmm = function(object, data, trait, ...) {
     if (is.null(data) && !is.null(object$data)) # allow for including data in object
         data = eval(object$data)
@@ -427,6 +431,7 @@ recover_data.MCMCglmm = function(object, data, trait, ...) {
 }
 
 # misc may be NULL or a list generated by trait spec
+#' @exportS3Method emm_basis MCMCglmm
 emm_basis.MCMCglmm = function(object, trms, xlev, grid, vcov., 
                               mode = c("default", "multinomial"), misc, ...) {
     nobs.MCMCglmm = function(object, ...) 1   # prevents warning about nobs
@@ -503,6 +508,7 @@ emm_basis.MCMCglmm = function(object, trms, xlev, grid, vcov.,
 ###    2. if it's right, even then
 ### Alternatively, maybe providing formula and data will do the trick
 
+#' @exportS3Method recover_data mcmc
 recover_data.mcmc = function(object, formula, data, ...) {
     if (missing(formula)) {
         cl = attr(object, "call")
@@ -521,6 +527,7 @@ recover_data.mcmc = function(object, formula, data, ...) {
     recover_data(cl, trms, NULL, data, ...)
 }
 
+#' @exportS3Method emm_basis mcmc         
 emm_basis.mcmc = function(object, trms, xlev, grid, vcov., contrasts.arg = NULL, ...) {
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts.arg)
@@ -538,10 +545,12 @@ emm_basis.mcmc = function(object, trms, xlev, grid, vcov., contrasts.arg = NULL,
 
 
 ### Support for mcmc.list
+#' @exportS3Method recover_data mcmc.list
 recover_data.mcmc.list = function(object, formula, data, ...) {
     recover_data.mcmc(object[[1]], formula, data, ...)
 }
 
+#' @exportS3Method emm_basis mcmc.list    
 emm_basis.mcmc.list = function(object, trms, xlev, grid, vcov., ...) {
     result = emm_basis.mcmc(object[[1]], trms, xlev, grid, vcov., ...)
     cols = seq_len(ncol(result$post.beta))
@@ -555,6 +564,7 @@ emm_basis.mcmc.list = function(object, trms, xlev, grid, vcov., ...) {
 
 ### support for CARBayes package - currently MUST supply data and have
 ### default contrasts matching what was used in fitting the mdoel
+#' @exportS3Method recover_data carbayes
 recover_data.carbayes = function(object, data, ...) {
     if(is.null(data)) # Try to recover data from parent frame
         data = model.frame(object$formula, data = parent.frame())
@@ -563,6 +573,7 @@ recover_data.carbayes = function(object, data, ...) {
     recover_data(cl, trms, NULL, data, ...)
 }
 
+#' @exportS3Method emm_basis carbayes
 emm_basis.carbayes = function(object, trms, xlev, grid, ...) {
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = attr(object$X, "contrasts"))
@@ -579,11 +590,13 @@ emm_basis.carbayes = function(object, trms, xlev, grid, ...) {
 
 ### Support for the rstanarm package (stanreg objects)
 ###
+#' @exportS3Method recover_data stanreg
 recover_data.stanreg = function(object, ...) {
     recover_data.lm(object, ...)
 }
 
 # note: mode and rescale are ignored for some models
+#' @exportS3Method emm_basis stanreg      
 emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
     misc = list()
     if (!is.null(object$family)) {
