@@ -123,13 +123,21 @@ print.emmGrid = function(x, ..., export = FALSE)
 #' 
 #' @param object An \code{emmGrid} object
 #' @param ... (required but not used)
+#' @param sep separator for pasting levels in creating row and column
+#'        names for \code{vcov()} results
 #' 
 #' @return The \code{vcov} method returns a symmetric matrix of variances and
 #'   covariances for \code{predict.emmGrid(object, type = "lp")}
 #'
 #' @method vcov emmGrid
 #' @export
-vcov.emmGrid = function(object, ...) {
+#' @examples
+#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
+#' warp.emm <- emmeans(warp.lm, ~ tension | wool)
+#' vcov(warp.emm) |> zapsmall()
+#' 
+#' vcov(pairs(warp.emm), sep = "|") |> zapsmall()
+vcov.emmGrid = function(object, ..., sep = get_emm_option("sep")) {
     tol = get_emm_option("estble.tol")
     if (!is.null(hook <- object@misc$vcovHook)) {
         if (is.character(hook)) 
@@ -137,11 +145,17 @@ vcov.emmGrid = function(object, ...) {
         hook(object, tol = tol, ...)
     }
     else {
-        X = object@linfct
+        if(is.null(disp <- object@misc$display))
+            disp = seq_len(nrow(object@linfct))
+        X = object@linfct[disp, , drop = FALSE]
         estble = estimability::is.estble(X, object@nbasis, tol) 
         X[!estble, ] = NA
         X = X[, !is.na(object@bhat), drop = FALSE]
-        X %*% tcrossprod(object@V, X)
+        rtn = X %*% tcrossprod(object@V, X)
+        largs = as.list(object@grid[disp, seq_along(object@levels), drop = FALSE])
+        largs$sep = sep
+        rownames(rtn) = colnames(rtn) = do.call(paste, largs)
+        rtn
     }
 }
 
@@ -221,6 +235,9 @@ vcov.emmGrid = function(object, ...) {
 #' \item{\code{adjust}}{(\code{character)}) is the default for the \code{adjust}
 #' argument in \code{\link{summary.emmGrid}}.}
 #' 
+#' \item{\code{cross.adjust}}{(\code{character)}) is the default for the \code{cross.adjust}
+#' argument in \code{\link{summary.emmGrid}} (used for adjusting between groups).}
+#' 
 #' \item{\code{famSize}}{(integer) is the number of means involved in a family of
 #' inferences; used in Tukey adjustment}
 #' 
@@ -254,7 +271,7 @@ vcov.emmGrid = function(object, ...) {
 #' \code{"link"} (with synonyms \code{"lp"} and \code{"linear"}), or
 #' \code{"response"}.}
 #' 
-#' \item{\code{bias.adjust}, \code{frequentist}}{(character) These
+#' \item{\code{bias.adjust}, \code{frequentist}}{(logical) These
 #' are used by \code{summary} if the value of these arguments are not specified.}
 #' 
 #' \item{\code{estType}}{(\code{character}) is used internally to determine 
@@ -453,7 +470,8 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 }
 
 ### List of valid strings to match in update() ###
-.valid.misc = c("adjust","alpha","avgd.over","bias.adjust","by.vars","calc","delta","df",
+.valid.misc = c("adjust","alpha","avgd.over","bias.adjust","by.vars","calc",
+                "cross.adjust", "delta","df",
                "initMesg","estName","estType","famSize","frequentist","infer","inv.lbl",
                "level","methDesc","nesting","null","predict.type","pri.vars",
                "side","sigma","tran","tran.mult","tran.offset","tran2","type","is.new.rg",
@@ -532,9 +550,9 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   \code{FALSE}, the system value \code{getOption("digits")} is used.}
 #' \item{\code{back.bias.adj}}{A logical value controlling whether we 
 #'   try to adjust bias when back-transforming. If \code{FALSE}, we use naive
-#'   back transformation. If \code{TRUE} \emph{and \code{sigma} is available}, a
+#'   back transformation. If \code{TRUE} \emph{and \code{sigma} is available and valid}, a
 #'   second-order adjustment is applied to estimate the mean on the response
-#'   scale.}
+#'   scale. A warning is issued if no valid \code{sigma} is available}
 #' \item{\code{enable.submodel}}{A logical value. If \code{TRUE}, enables support 
 #'   for selected model classes to implement the \code{submodel} option. If
 #'   \code{FALSE}, this support is disabled. Setting this option to \code{FALSE}
@@ -892,12 +910,12 @@ emm_defaults = list (
 #'   when one specifies \code{transform = "log"} but desires summaries to be
 #'   produced by default on the response scale.
 #' @param bias.adjust Logical value for whether to adjust for bias in
-#'   back-transforming (\code{transform = "response"}). This requires a value of 
+#'   back-transforming (\code{transform = "response"}). This requires a valid value of 
 #'   \code{sigma} to exist in the object or be specified.
 #' @param sigma Error SD assumed for bias correction (when 
 #'   \code{transform = "response"} and a transformation
 #'   is in effect). If not specified,
-#'   \code{object@misc$sigma} is used, and an error is thrown if it is not found.
+#'   \code{object@misc$sigma} is used, and a warning is issued if it is not found.
 #' @param N.sim Integer value. If specified and \code{object} is based on a 
 #'   frequentist model (i.e., does not have a posterior sample), then a fake 
 #'   posterior sample is generated using the function \code{sim}.
@@ -997,7 +1015,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
         pargs = object@grid[names(object@levels)]
         lbls = do.call(paste, c(pargs, sep = "."))
         if (!is.null(disp <- object@misc$display)) {  # fix up for the bookkeeping in nested models
-            object@V = object@V[disp, disp, drop = FALSE]
+            object@V = object@V[estble, estble, drop = FALSE]
             object@linfct = matrix(0, nrow = length(disp), ncol = length(estble))
             object@linfct[disp, ] = diag(1, length(estble))
             lbls = lbls[disp]
@@ -1040,6 +1058,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
             if(missing(sigma))
                 sigma = object@misc$sigma
             link = .make.bias.adj.link(link, sigma)
+            #### (not needed)   bias.adjust = attr(link, "bias.adjust")
             if (!is.na(PB[1])) # special frequentist version when sigma is MCMC sample
                 flink = .make.bias.adj.link(flink, mean(sigma))
             else
