@@ -26,8 +26,8 @@
 #' This function may make it possible to compute a reference grid for a model 
 #' object that is otherwise not supported.
 #' 
-#' Usually, you need to provide 
-#' \code{formula}, \code{data}, \code{coef}, and \code{vcov}, and perhaps other
+#' Usually, you need to provide either \code{object}; or
+#' \code{formula}, \code{coef}, \code{vcov}, \code{data}, and perhaps other
 #' parameters. It is usually fairly straightforward to figure out how to get
 #' these from the model \code{object}; see the documentation for the model class that
 #' was fitted. Sometimes one or more of these quantities contains extra parameters,
@@ -39,23 +39,22 @@
 #' the total number of elements of \code{coef}.)
 #' 
 #' If your model object follows fairly closely the conventions of an \code{\link[stats]{lm}}
-#' object, you may be able to get by providing the model as \code{object}, plus and (probably) \code{data},
+#' or \code{\link[stats]{glm}}object, you may be able to get by providing the model as \code{object},
 #' and perhaps some other parameters to override the defaults.
 #' When \code{object} is specified, it is used as detailed below to try to obtain the 
 #' other arguments. The user should ensure that the defaults
 #' shown below do indeed work. 
-#' 
 #' The default values for the arguments are as follows:
 #' \itemize{
-#'   \item{\code{formula}: }{Required unless obtainable via \code{formula(object)}}
-#'   \item{\code{data}: }{Required if variables are not in \code{parent.frame()} or 
-#'       obtainable via \code{object$data}}
-#'   \item{\code{coef}: }{\code{coef(object)}}
-#'   \item{\code{vcov}: }{\code{vcov(object)}}
-#'   \item{\code{df}: }{Set to \code{Inf} if not available in \code{object$df.residual}}
-#'   \item{\code{mcmc}: }{\code{object$sample}}
-#'   \item{\code{subset}: }{\code{NULL} (so that all observations in \code{data} are used)}
-#'   \item{\code{contrasts}: }{\code{object$contrasts}}
+#'   \item{\code{formula}: \code{formula(object)}}
+#'   \item{\code{data}: \code{recover_data.lm(object)} is tried, and if an error is thrown,
+#'     we also check \code{object$data}.}
+#'   \item{\code{coef}: \code{coef(object)}}
+#'   \item{\code{vcov}: \code{vcov(object)}}
+#'   \item{\code{df}: Set to \code{Inf} if not available in \code{df.residual(object)}}
+#'   \item{\code{mcmc}: \code{object$sample}}
+#'   \item{\code{subset}: \code{NULL} (so that all observations in \code{data} are used)}
+#'   \item{\code{contrasts}: \code{object$contrasts}}
 #' }
 #' 
 #' The functions \code{\link{qdrg}} and \code{emmobj} are close cousins, in that
@@ -80,14 +79,17 @@
 #'     (Note: response transformations are auto-detected from \code{formula})
 #' @param qr QR decomposition of the model matrix; used only if there are \code{NA}s
 #'     in \code{coef}.
-#' @param ordinal.dim Integer number of levels in an ordinal response. If not
-#'     missing, the intercept terms are modified appropriate to predicting the latent
-#'     response (see \code{vignette("models")}, Section O, but note that the other 
-#'     modes, e.g., \code{mode = "prob"} are not provided). 
-#'     In this case, we expect
-#'     the first \code{ordinal.dim - 1} elements of \code{coef} to be the
+#' @param ordinal \code{list} with elements \code{dim} and \code{mode}.
+#'     \code{ordinal$dim} (integer) is the number of levels in an ordinal response. If 
+#'     \code{ordinal} is provided, the intercept terms are modified appropriate to predicting 
+#'     an ordinal response, as described in \code{vignette("models")}, Section O,
+#'     using \code{ordinal$mode} as the \code{mode} argument (if not
+#'     provided, \code{"latent"} is assumed).
+#'     (All modes are supported except `scale`)
+#'     For this to work, we expect
+#'     the first \code{ordinal$dim - 1} elements of \code{coef} to be the
 #'     estimated threshold parameters, followed by the coefficients for the
-#'     linear predictor.) 
+#'     linear predictor.
 #' @param ... Optional arguments passed to \code{\link{ref_grid}}
 #'
 #' @return An \code{emmGrid} object constructed from the arguments
@@ -106,7 +108,9 @@
 #' 
 #' @seealso \code{\link{emmobj}} for an alternative way to construct an \code{emmGrid}.
 #' 
-#' 
+#' @note For backwards compatibility, an argument \code{ordinal.dim} is invisibly 
+#' supported as part of \code{...}, and if present, sets 
+#' \code{ordinal = list(dim = ordinal.dim, mode = "latent")}
 #' 
 #' @export
 #' @examples
@@ -122,13 +126,22 @@
 #'     emm_example("qdrg-ordinal")
 #'
 qdrg = function(formula, data, coef, vcov, df, mcmc, object,
-                subset, weights, contrasts, link, qr, ordinal.dim, ...) {
+                subset, weights, contrasts, link, qr, ordinal, ...) {
+    # back-compatible access to old ordinal.dim arg...
+    od = (\(ordinal, ordinal.dim = NULL, ...) {
+        if(!missing(ordinal) && is.numeric(ordinal)) ordinal.dim = ordinal
+        ordinal.dim
+    })(ordinal, ...)
+    if(!is.null(od)) ordinal = list(dim = od, mode = "latent")
+    
     result = match.call(expand.dots = FALSE)
     if (!missing(object)) {
         if (missing(formula)) 
             result$formula = stats::formula(object)
         if (missing(data)) {
-            data = object$data
+            data = try(recover_data.lm(object), silent = TRUE)
+            if(inherits(data, "try-error"))
+                data = object$data
             if (is.null(data)) data = parent.frame()
             result$data = data
         }
@@ -139,7 +152,7 @@ qdrg = function(formula, data, coef, vcov, df, mcmc, object,
         if (missing(vcov)) 
             result$vcov = stats::vcov(object)
         if(missing(df))
-            result$df = object$df.residual
+            result$df = df.residual(object)
         if(missing(contrasts))
             result$contrasts = object$contrasts
         if(any(is.na(result$coef)))
@@ -157,6 +170,7 @@ qdrg = function(formula, data, coef, vcov, df, mcmc, object,
         if (!missing(vcov)) result$vcov = vcov
         if(!missing(df)) result$df = df
     }
+    if(!missing(df)) result$df = df
     if(is.null(result$df))
         result$df = Inf
     if(!missing(mcmc)) result$mcmc = mcmc
@@ -165,14 +179,14 @@ qdrg = function(formula, data, coef, vcov, df, mcmc, object,
     if(!missing(contrasts)) result$contrasts = contrasts
     if(!missing(link)) result$link = link
     if(!missing(qr) && any(is.na(result$coef))) result$qr = qr
-    if(!missing(ordinal.dim)) result$ordinal.dim = ordinal.dim
+    if(!missing(ordinal)) result$ordinal = ordinal
     
     # make sure "formula" exists, has a LHS and is is 2nd element so that 
     # response transformation can be found
     if (is.null(result$formula))
         stop("No formula; cannot construct a reference grid")
     if(length(result$formula) < 3)
-        result$formula = update(result$formula, .dummy ~ .)
+        result$formula = update.formula(result$formula, response ~ .)
     fpos = grep("formula", names(result))[1]
     result = result[c(1, fpos, seq_along(result)[-c(1, fpos)])]
 
@@ -180,6 +194,7 @@ qdrg = function(formula, data, coef, vcov, df, mcmc, object,
     ref_grid(result, ...)
 }
 
+#' @exportS3Method recover_data qdrg
 recover_data.qdrg = function(object, ...) {
     recover_data.call(object, delete.response(terms(object$formula)), object$na.action, ...)
 }
@@ -187,6 +202,7 @@ recover_data.qdrg = function(object, ...) {
 vcov.qdrg = function(object, ...) 
     object$vcov
 
+#' @exportS3Method emm_basis qdrg         
 emm_basis.qdrg = function(object, trms, xlev, grid, ...) {
     m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
     X = model.matrix(trms, m, contrasts.arg = object$contrasts)
@@ -199,11 +215,32 @@ emm_basis.qdrg = function(object, trms, xlev, grid, ...) {
         if (is.null(object$vcov)) V = cov(object$mcmc)
     }
     
+    misc = list()
+    
     # If ordinal, add extra avgd, subtracted intercepts -- for latent mode
-    if(!is.null(od <- object$ordinal.dim)) { 
-        intcpt = matrix(-1 / (od - 1), nrow = nrow(X), ncol = od - 1)
-        colnames(intcpt) = names(bhat)[1:(od-1)]
-        X = cbind(intcpt, X[, -1, drop = FALSE])
+    if(!is.null(ordinal <- object$ordinal)) {
+        if(is.null(ordinal$mode)) ordinal$mode = "latent"
+        ordinal$mode = match.arg(ordinal$mode, c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"))
+        if(is.null(od <- ordinal$dim)) stop ("'ordinal' MUST have a 'dim' element", call. = FALSE)
+        if(ordinal$mode == "latent") {
+            intcpt = matrix(-1 / (od - 1), nrow = nrow(X), ncol = od - 1)
+            colnames(intcpt) = names(bhat)[1:(od - 1)]
+            X = cbind(intcpt, X[, -1, drop = FALSE])
+        }
+        else {
+            levs = seq_len(od - 1)
+            misc$ylevs = list(cut = paste(levs, levs+1, sep = "|"))
+            misc$inv.lbl = "cumprob"
+            misc$offset.mult = -1
+            J = matrix(1, nrow = od - 1)
+            X = cbind(kronecker(diag(od - 1), matrix(1, nrow = nrow(X))), 
+                      kronecker(-J, X[, -1, drop = FALSE]))
+            if (ordinal$mode != "linear.predictor") {
+                misc$mode = ordinal$mode
+                misc$respName = as.character(object$formula)[2]
+                misc$postGridHook = ".clm.postGrid"
+            }
+        }
     }
     
     bhat = .impute.NAs(bhat, X) # make coefs lm-compatible
@@ -217,14 +254,12 @@ emm_basis.qdrg = function(object, trms, xlev, grid, ...) {
             mm = suppressWarnings(model.frame(trms, object$data, na.action = na.pass, xlev = xlev))
             XX = model.matrix(trms, mm, contrasts.arg = object$contrasts)
             nbasis = estimability::nonest.basis(XX)
-            if (nrow(V) == length(bhat)) {
-                ii = which(!is.na(bhat))
-                V = V[ii, ii, drop = FALSE]
-            }
         }
-     }
-    
-    misc = list()
+        if (nrow(V) == length(bhat)) {
+            ii = which(!is.na(bhat))
+            V = V[ii, ii, drop = FALSE]
+        }
+    }
     
     # check multivariate situation
     if (is.matrix(bhat)) {
